using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Collections.Specialized;
using System.IO;

namespace Mask_Map {
    public partial class Test : Form {

        private class ImagePair {
            public string imageFile;
            public string maskFile;            
            public ImagePair(string imageFile, string maskFile) {
                this.imageFile = imageFile;
                this.maskFile = maskFile;
            }
            public override string ToString() {
                return string.Format("{2} - {0} ({1})", Path.GetFileNameWithoutExtension(this.imageFile), Path.GetFileNameWithoutExtension(this.maskFile), this.imageFile.ToLowerInvariant().Contains("bad") ? "Bad" : "Good");
            }
        }

        public Test() {
            InitializeComponent();
            
            TestCombo.Items.Add(new ImagePair(@"..\good\Image 00418.bmp", @"..\good\Image 00434.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\good\Image 00450.bmp", @"..\good\Image 00452.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00436.bmp", @"..\bad\Image 00435.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00438.bmp", @"..\bad\Image 00440.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00439.bmp", @"..\bad\Image 00441.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00451.bmp", @"..\bad\Image 00453.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00458.bmp", @"..\bad\Image 00460.bmp"));
            TestCombo.Items.Add(new ImagePair(@"..\bad\Image 00459.bmp", @"..\bad\Image 00461.bmp"));
            TestCombo.SelectedIndex = 0;
        }


        // Holds the testing image
        private Bitmap testImage;
        // Holds the mask for said testing image
        private Bitmap testMask;

        private void UpdateStages() {
            if (this.TestOutput.Image != null) this.TestOutput.Image.Dispose();
            this.TestOutput.Image = DrawStages(testImage, testMask, this.TestTrackbar.Value);
        }


        private Bitmap DrawStages(Bitmap image, Bitmap mask, int amount) {

            int w = image.Width;
            int h = image.Height;

            if (mask.Width != w || mask.Height != h) throw new Exception("Image and mask must be the same size");


            Rectangle entireImage = new Rectangle(0,0,w,h);
            BitmapData bd;            
            
            bd = image.LockBits(entireImage, ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            int imageStride = bd.Stride / 4;
            int[] imageArray = new int[imageStride * h];
            Marshal.Copy(bd.Scan0, imageArray, 0, imageStride * h);
            image.UnlockBits(bd);

            
            bd = mask.LockBits(entireImage, ImageLockMode.ReadOnly, PixelFormat.Format8bppIndexed);
            int maskStride = bd.Stride;
            byte[] maskArray = new byte[maskStride * h];
            Marshal.Copy(bd.Scan0, maskArray, 0, maskStride * h);
            mask.UnlockBits(bd);


            for (int y = 0; y < h; ++y) {
                int imageI = y * imageStride;
                int maskI = y * maskStride;
                for (int x = 0; x < w; ++x) {
                    if (maskArray[maskI++] >= amount) imageArray[imageI + x] = 0;
                }
                
            }

            Bitmap output = new Bitmap(w, h, PixelFormat.Format32bppArgb);
            bd = output.LockBits(entireImage, ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(imageArray, 0, bd.Scan0, w * h);
            output.UnlockBits(bd);

            return output;

        }

        private void TestTrackbar_Scroll(object sender, EventArgs e) {
            UpdateStages();
        }

        private void TestCombo_SelectedIndexChanged(object sender, EventArgs e) {
            if (testImage != null) testImage.Dispose();
            if (testMask != null) testMask.Dispose();

            ImagePair ip = (ImagePair)TestCombo.SelectedItem;
            testImage = new Bitmap(ip.imageFile);
            testMask = new Bitmap(ip.maskFile);

            // This clears the bluey-purple background colour to transparent.
            testImage.MakeTransparent(Color.FromArgb(107, 107, 191));

            UpdateStages();
        }

    }
}